<?php 

if ( ! defined( 'ABSPATH' ) )
	exit;

if ( !class_exists( 'ewdHandleUpgrades' ) ) {
class ewdHandleUpgrades {

	// The plugin that is being manipulated
	public $plugin = ''; 

	// The product key entered into the upgrade form
	public $key = '';

	private $permission_level = 2;

	private $controller_variable_name = '';

	private $full_variable_name = '';

	private $full_message_name = '';

	public function __construct() {

		if ( isset( $_POST['ewd_upgrade_to_full'] ) ) {

			$this->plugin = $_POST['plugin_name'];
			$this->key = trim($_POST['key']);

			$this->set_private_variables();

			add_action( 'admin_init', array( $this, 'ewd_upgrade_to_full') );
		}
	}

 	public function ewd_upgrade_to_full() {
		
		if ( $this->key == "EWD Trial" and ! $this->is_trial_happening() ) { 
			
			$this->set_message( 'Update', __("Trial successfully started!", 'ewd-premium-helper') );

			$this->set_trial_options();

			$this->update_version_status();

			$this->send_trial_information();
		}
		elseif ( strlen( $this->key ) < 18 or strlen( $this->key ) > 22 ) {

			$this->set_message( 'Error', __("Invalid Product Key", 'ewd-premium-helper') );
		}
		elseif ( $this->key != "EWD Trial" ) {
			
			$premium_response = $this->check_product_key();
			
			if ( $premium_response['Message_Type'] == "Error" && false == $premium_response['bypass'] ) {
				
				$this->set_message( "Error", $premium_response['Message'] );
			}
			else {

				if( true == $premium_response['bypass'] ) {
					wp_mail(
						'contact@etoilewebdesign.com', 
						"{$this->plugin} activation server communication", 
						"This server is unable to connect to the Etoile Web Design server for activation.\n\nThe license key is: $this->key\n\nWebsite: " . get_bloginfo( 'name' )
					);
				}

				$this->set_message( "Update", $premium_response['Message'] );

				$this->permission_level = isset( $premium_response['Permission_Level'] ) ? $premium_response['Permission_Level'] : 2;

				$this->do_upgrade();
			}
		}
	}

	public function set_private_variables() {

		$this->controller_variable_name = 'ewd_' . strtolower( $this->plugin ) . '_controller';
		$this->full_variable_name = $this->plugin == 'UPCP' ? 'Full_Version' : 'EWD_' . $this->plugin . '_Full_Version';
		$this->full_message_name = ( $this->plugin == 'UPCP' or $this->plugin == 'FEUP' ) ? strtolower( $this->plugin ) . '_message' : 'ewd_' . strtolower( $this->plugin ) . '_message';
	}

	public function is_trial_happening() {

		return get_option( 'EWD_' . $this->plugin . '_Trial_Happening' ) != '' ? true : false;
	}

	public function controller_exists() {
		global ${$this->controller_variable_name};

		return ( ! empty( ${$this->controller_variable_name} ) and is_object( ${$this->controller_variable_name} ) );
	}

	public function set_message( $type, $message ) {
		global ${$this->full_message_name};

		${$this->full_message_name} = array( "Message_Type" => $type, "Message" => $message );
	}

	public function set_trial_options() {

		if ( $this->controller_exists() ) {
				update_option( 'EWD_' . $this->plugin .'_Trial_Expiry_Time', time() + (7*24*60*60) );
				update_option( 'EWD_' . $this->plugin . '_Trial_Happening', 'Yes');
				update_option( 'ewd-' . strtolower( $this->plugin ) . '-permission-level', 2 );
		}
		else {
			
			update_option( 'EWD_' . $this->plugin .'_Trial_Expiry_Time', time() + (7*24*60*60) );
			update_option( 'EWD_' . $this->plugin . '_Trial_Happening', 'Yes');

			if ( $this->plugin == 'UPCP' ) {
				
				update_option( $this->plugin . '_Full_Version', 'Yes');
			}
			else {
				
				update_option( 'EWD_' . $this->plugin . '_Full_Version', 'Yes');
			}
		}
	}

	public function update_version_status() {
		global ${$this->full_variable_name};
		global ${$this->controller_variable_name};

		if ( $this->controller_exists() ) { ${$this->controller_variable_name}->permissions->update_permissions(); }
		else { ${$this->full_variable_name} = $this->plugin == 'UPCP' ? get_option( $this->plugin .'_Full_Version' ) : get_option( 'EWD_' . $this->plugin .'_Full_Version' ); }
	}

	public function send_trial_information() {

		$admin_email = get_option( 'admin_email' );

		$response = wp_remote_get( 'http://www.etoilewebdesign.com/UPCP-Key-Check/Register_Trial.php?Plugin=' . $this->plugin . '&Admin_Email=' . $admin_email . '&Site=' . get_bloginfo( 'wpurl' ) );
	}

	public function check_product_key() {

		$theme = wp_get_theme();
		$theme_name = $theme->get( 'Name' );

		$response = wp_remote_get( 'http://www.etoilewebdesign.com/UPCP-Key-Check/EWD_' . $this->plugin . '_KeyCheck.php?Key=' . $this->key . '&Site=' . get_bloginfo( 'wpurl' ) . '&Theme_Name=' . $theme_name );
		
		if($response instanceof WP_Error) {
			if(defined('WP_DEBUG') && WP_DEBUG && defined('WP_DEBUG_LOG') && WP_DEBUG_LOG) {
				error_log('EWD Premium Helper ('.$response->get_error_code().') : '.$response->get_error_message());
			}

			return array(
				'Message_Type' => 'Error',
				'Message' => 'Unable to connect to Etoile Web Design servers. If the plugin does not activate, please contact our support team at contact@etoilewebdesign.com.',
				'bypass' => true
			);
		}

		$response = unserialize( wp_remote_retrieve_body( $response ) );
		$response['bypass'] = false;

		return $response;
	}

	public function do_upgrade() {

		delete_option( 'EWD_' . $this->plugin . '_Trial_Expiry_Time' );
		if ( $this->controller_exists() ) {
			update_option( 'EWD_' . $this->plugin . '_Trial_Happening', 'No' );
			update_option( 'ewd-' .strtolower( $this->plugin ) . '-permission-level', array( $this->permission_level ) );
		}
		else {
			update_option( 'EWD_' . $this->plugin . '_Trial_Happening', 'No' );
			update_option( 'EWD_' . $this->plugin . '_Full_Version', 'Yes' );
		}
	
		$this->update_version_status();
	
		update_option( 'EWD_' . $this->plugin . '_License_Key', $this->key );
		if ( $this->permission_level == 3 ) { update_option( strtolower( $this->plugin ) . '-ultimate-license-key', $this->key ); }
	}
}
}

 ?>
