<?php
if ( ! defined( 'ABSPATH' ) ) exit;

if ( !class_exists( 'ewduaspEWDPHSettings' ) ) {
class ewduaspEWDPHSettings {

  public function __construct() {

    add_filter( 'ewd_uasp_defaults', array( $this, 'ewd_uasp_settings_set_defaults' ), 10, 2 );

    add_filter( 'ewd_uasp_settings_page', array( $this, 'ewd_uasp_settings__non_premium_tabs' ), 10, 2 );

    add_filter( 'ewd_uasp_settings_page', array( $this, 'ewd_uasp_settings_premium_tab' ), 10, 2 );
    add_filter( 'ewd_uasp_settings_page', array( $this, 'ewd_uasp_settings_custom_fields_tab' ), 10, 2 );
    add_filter( 'ewd_uasp_settings_page', array( $this, 'ewd_uasp_settings_payments_tab' ), 10, 2 );
    add_filter( 'ewd_uasp_settings_page', array( $this, 'ewd_uasp_settings_emails_tab' ), 10, 2 );
    add_filter( 'ewd_uasp_settings_page', array( $this, 'ewd_uasp_settings_labelling_tab' ), 10, 2 );
    add_filter( 'ewd_uasp_settings_page', array( $this, 'ewd_uasp_settings_styling_tab' ), 10, 2 );
  }

  public function get_permission( $permission_type = '' ) {
    global $ewd_uasp_controller;
  
    $ewd_uasp_premium_permissions = array();
  
    if ( ! $ewd_uasp_controller->permissions->check_permission( $permission_type ) ) {
      $ewd_uasp_premium_permissions = array(
        'disabled'    => true,
        'disabled_image'=> '#',
        'purchase_link' => 'https://www.etoilewebdesign.com/plugins/ultimate-appointment-scheduling/?utm_source=uasp_lockbox'
      );
    }
  
    return $ewd_uasp_premium_permissions;
  }

  public function ewd_uasp_settings_set_defaults( $defaults, $ewduaspSettings ) {

    $defaults = array_merge(
      $defaults,
      array(
        // Any default which you are certain that won't be used for free version.
      )
    );
  
    return $defaults;
  }

  public function ewd_uasp_settings__non_premium_tabs( $sap, $ewduaspSettings ) {

    return $sap;
  }

  // "Premium" Tab
  public function ewd_uasp_settings_premium_tab( $sap, $ewduaspSettings ) {
    
    $sap->add_section(
      'ewd-uasp-settings',
      array_merge(
        array(
          'id'            => 'ewd-uasp-premium-general',
          'title'         => __( 'General', 'ultimate-appointment-scheduling' ),
          'tab'         => 'ewd-uasp-premium-tab',
        ),
        $this->get_permission( 'premium' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-premium-general',
      'radio',
      array(
        'id'      => 'booking-form-style',
        'title'     => __( 'Booking Form Style', 'ultimate-appointment-scheduling' ),
        'description' => __( 'What style should the booking form use?', 'ultimate-appointment-scheduling' ),
        'options'   => array(
          'standard'    => __( 'Standard', 'ultimate-appointment-scheduling' ),
          'contemporary'  => __( 'Contemporary', 'ultimate-appointment-scheduling' ),
        )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-premium-general',
      'toggle',
      array(
        'id'      => 'use-captcha',
        'title'     => __( 'Require Captcha', 'ultimate-appointment-scheduling' ),
        'description' => __( 'Should a captcha field be added to your appointment scheduling form?', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-premium-general',
      'radio',
      array(
        'id'      => 'captcha-type',
        'title'     => __( 'Captcha Type', 'ultimate-appointment-scheduling' ),
        'description' => sprintf( __( 'Plain text will soon be deprecated as it is easier to bypass. Google\'s reCAPTCHA code to your form, to verify guests before they can book. Please check %s our documentation %s for more information on how to configure this feature.', 'ultimate-appointment-scheduling' ), '<a href="https://www.etoilewebdesign.com/2021/09/02/setting-up-recatpcha-in-the-ultimate-reviews-plugin/" target="_blank">', '</a>'),
        'default'   => $ewduaspSettings->defaults['captcha-type'],
        'options'   => array(
          'text'      => __( 'Plain Text', 'ultimate-appointment-scheduling' ),
          'recaptcha'   => __( 'Google ReCAPTCHA', 'ultimate-appointment-scheduling' ),
        ),
        'conditional_on'    => 'use-captcha',
        'conditional_on_value'  => true
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-premium-general',
      'text',
      array(
        'id'            => 'captcha-site-key',
        'title'         => __( 'Google Site Key', 'ultimate-appointment-scheduling' ),
        'description'   => __( 'The site key provided to you by Google', 'ultimate-appointment-scheduling' ),
        'conditional_on'    => 'captcha-type',
        'conditional_on_value'  => 'recaptcha'
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-premium-general',
      'text',
      array(
        'id'            => 'captcha-secret-key',
        'title'         => __( 'Google Secret Key', 'ultimate-appointment-scheduling' ),
        'description'   => __( 'The secret key provided to you by Google', 'ultimate-appointment-scheduling' ),
        'conditional_on'    => 'captcha-type',
        'conditional_on_value'  => 'recaptcha'
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-premium-general',
      'select',
      array(
        'id'            => 'access-role',
        'title'         => __( 'Set Access Role', 'ultimate-appointment-scheduling' ),
        'description'   => __( 'Who should have access to the "Appointments" admin menu?', 'ultimate-appointment-scheduling' ), 
        'blank_option'  => false,
        'options'       => array(
          'administrator'       => __( 'Administrator', 'ultimate-appointment-scheduling' ),
          'delete_others_pages'   => __( 'Editor', 'ultimate-appointment-scheduling' ),
          'delete_published_posts'  => __( 'Author', 'ultimate-appointment-scheduling' ),
          'delete_posts'        => __( 'Contributor', 'ultimate-appointment-scheduling' ),
        )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-premium-general',
      'toggle',
      array(
        'id'      => 'require-login',
        'title'     => __( 'Require Login', 'ultimate-appointment-scheduling' ),
        'description' => __( 'Do visitors have to login before they can book an appointment?', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-premium-general',
      'text',
      array(
        'id'            => 'wordpress-login-url',
        'title'         => __( 'WordPress Login URL', 'ultimate-appointment-scheduling' ),
        'description' => __( 'The URL of your WordPress login page, if different from the expected page.', 'ultimate-appointment-scheduling' ),
        'conditional_on'    => 'require-login',
        'conditional_on_value'  => true
      )
    );
  
    return $sap;
  }

  // "Custom Fields" Tab
  public function ewd_uasp_settings_custom_fields_tab( $sap, $ewduaspSettings ) {
    
    $sap->add_section(
      'ewd-uasp-settings',
      array_merge(
        array(
          'id'            => 'ewd-uasp-custom-fields',
          'title'         => __( 'Custom Fields', 'ultimate-appointment-scheduling' ),
          'tab'         => 'ewd-uasp-custom-fields-tab',
        ),
        $this->get_permission( 'custom_fields' )
      )
    );

    $fields_description = __( 'Should any extra fields be added to the booking form?', 'ultimate-appointment-scheduling' ) . '<br />';
    $fields_description .= __( 'The "Field Values" should be a comma-separated list of values for the select, radio or checkbox field types (no extra spaces after the comma).', 'ultimate-appointment-scheduling' ) . '<br />';
    
    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-custom-fields',
      'infinite_table',
      array(
        'id'      => 'custom-fields',
        'title'     => __( 'Custom Fields', 'ultimate-appointment-scheduling' ),
        'add_label'   => __( '+ ADD', 'ultimate-appointment-scheduling' ),
        'del_label'   => __( 'Delete', 'ultimate-appointment-scheduling' ),
        'description' => $fields_description,
        'fields'    => array(
          'id' => array(
            'type'    => 'hidden',
            'label'   => 'Field ID',
            'required'  => true
          ),
          'name' => array(
            'type'    => 'text',
            'label'   => 'Field Name',
            'required'  => true
          ),
          
          'slug' => array(
            'type'    => 'text',
            'label'   => 'Field Slug',
            'required'  => true
          ),
          'type' => array(
            'type'    => 'select',
            'label'   => __( 'Field Type', 'ultimate-appointment-scheduling' ),
            'options'   => array(
              'text'    => __( 'Text', 'ultimate-appointment-scheduling' ),
              'number'  => __( 'Number', 'ultimate-appointment-scheduling' ),
              'textarea'  => __( 'Text Area', 'ultimate-appointment-scheduling' ),
              'select'  => __( 'Select Box', 'ultimate-appointment-scheduling' ),
              'radio'   => __( 'Radio Buttons', 'ultimate-appointment-scheduling' ),
              'checkbox'  => __( 'Checkbox', 'ultimate-appointment-scheduling' ),
              'link'    => __( 'Link', 'ultimate-appointment-scheduling' ),
              'date'    => __( 'Date', 'ultimate-appointment-scheduling' ),
              'datetime'  => __( 'Date/Time', 'ultimate-appointment-scheduling' ),
            )
          ),
          'options' => array(
            'type'    => 'text',
            'label'   => __( 'Field Values', 'ultimate-appointment-scheduling' ),
            'required'  => false
          )
        )
      )
    );

    return $sap;
  }

  // "Payments" Tab
  public function ewd_uasp_settings_payments_tab( $sap, $ewduaspSettings ) {
    
    $sap->add_section(
      'ewd-uasp-settings',
      array_merge(
        array(
          'id'            => 'ewd-uasp-payment-woocommerce',
          'title'         => __( 'WooCommerce Payment Options', 'ultimate-appointment-scheduling' ),
          'tab'         => 'ewd-uasp-payments-tab',
        ),
        $this->get_permission( 'payments' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-payment-woocommerce',
      'toggle',
      array(
        'id'      => 'woocommerce-integration',
        'title'     => __( 'Require WooCommerce Payment', 'ultimate-appointment-scheduling' ),
        'description' => __( 'Should services be synced as WooCommerce products, and customers required to checkout before scheduling an appointment?', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_section(
      'ewd-uasp-settings',
      array_merge(
        array(
          'id'            => 'ewd-uasp-payment-paypal',
          'title'         => __( 'PayPal Payment Options', 'ultimate-appointment-scheduling' ),
          'tab'         => 'ewd-uasp-payments-tab',
        ),
        $this->get_permission( 'payments' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-payment-paypal',
      'radio',
      array(
        'id'      => 'paypal-prepayment',
        'title'     => __( 'Allow Prepayment through PayPal', 'ultimate-appointment-scheduling' ),
        'description' => __( 'Should customers be required to, able to (optional) or unable to pay for their appointments through PayPal when they book them?', 'ultimate-appointment-scheduling' ),
        'options'   => array(
          'required'    => __( 'Required', 'ultimate-appointment-scheduling' ),
          'optional'    => __( 'Optional', 'ultimate-appointment-scheduling' ),
          'none'      => __( 'None', 'ultimate-appointment-scheduling' ),
        )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-payment-paypal',
      'select',
      array(
        'id'            => 'currency-code',
        'title'         => __( 'Pricing Currency', 'ultimate-appointment-scheduling' ),
        'description'   => __( 'What currency are your services priced in?', 'ultimate-appointment-scheduling' ), 
        'blank_option'  => false,
        'options'       => $ewduaspSettings->currency_options
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-payment-paypal',
      'text',
      array(
        'id'            => 'paypal-email-address',
        'title'         => __( 'PayPal Email Address', 'ultimate-appointment-scheduling' ),
        'description' => __( 'If PayPal payments are required or optional, what email address is associated with the PayPal account?', 'ultimate-appointment-scheduling' ),
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-payment-paypal',
      'text',
      array(
        'id'            => 'thank-you-url',
        'title'         => __( '"Thank You" Page URL', 'ultimate-appointment-scheduling' ),
        'description' => __( 'What page should customers be taken to after successfully completing a PayPal payment?', 'ultimate-appointment-scheduling' ),
      )
    );

    return $sap;
  }

  // "Emails" Tab - Styling Section
  public function ewd_uasp_settings_emails_tab( $sap, $ewduaspSettings ) {
    
    $sap->add_section(
      'ewd-uasp-settings',
      array_merge(
        array(
          'id'            => 'ewd-uasp-email-styling',
          'title'         => __( 'Reminder Email Styling', 'ultimate-appointment-scheduling' ),
          'tab'         => 'ewd-uasp-emails-tab',
        ),
        $this->get_permission( 'styling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-email-styling',
      'colorpicker',
      array(
        'id'      => 'styling-email-background-color',
        'title'     => __( 'Email Background Color', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-email-styling',
      'colorpicker',
      array(
        'id'      => 'styling-email-inner-color',
        'title'     => __( 'Email Inner Background Color', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-email-styling',
      'colorpicker',
      array(
        'id'      => 'styling-email-text-color',
        'title'     => __( 'Email Text Color', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-email-styling',
      'colorpicker',
      array(
        'id'      => 'styling-email-button-bg-color',
        'title'     => __( 'Email Button Background Color', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-email-styling',
      'colorpicker',
      array(
        'id'      => 'styling-email-button-text-color',
        'title'     => __( 'Email Button Text Color', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-email-styling',
      'colorpicker',
      array(
        'id'      => 'styling-email-button-bg-hover-color',
        'title'     => __( 'Email Button Hover Background Color', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-email-styling',
      'colorpicker',
      array(
        'id'      => 'styling-email-button-text-hover-color',
        'title'     => __( 'Email Button Hover Text Color', 'ultimate-appointment-scheduling' )
      )
    );

    return $sap;
  }

  // "Labelling" Tab
  public function ewd_uasp_settings_labelling_tab( $sap, $ewduaspSettings ) {
    
    $sap->add_section(
      'ewd-uasp-settings',
      array_merge(
        array(
          'id'            => 'ewd-uasp-labelling-sign-up',
          'title'         => __( 'Sign Up', 'ultimate-appointment-scheduling' ),
          'tab'         => 'ewd-uasp-labelling-tab',
        ),
        $this->get_permission( 'labelling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-labelling-sign-up',
      'text',
      array(
        'id'            => 'label-sign-up-title',
        'title'         => __( '\'Sign Up\' Title', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-labelling-sign-up',
      'text',
      array(
        'id'            => 'label-name',
        'title'         => __( 'Name', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-labelling-sign-up',
      'text',
      array(
        'id'            => 'label-phone',
        'title'         => __( 'Phone', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-labelling-sign-up',
      'text',
      array(
        'id'            => 'label-email',
        'title'         => __( 'Email', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_section(
      'ewd-uasp-settings',
      array_merge(
        array(
          'id'            => 'ewd-uasp-labelling-service',
          'title'         => __( 'Service', 'ultimate-appointment-scheduling' ),
          'tab'         => 'ewd-uasp-labelling-tab',
        ),
        $this->get_permission( 'labelling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-labelling-service',
      'text',
      array(
        'id'            => 'label-service-title',
        'title'         => __( '\'Service\' Title', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-labelling-service',
      'text',
      array(
        'id'            => 'label-location',
        'title'         => __( 'Location', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-labelling-service',
      'text',
      array(
        'id'            => 'label-service',
        'title'         => __( 'Service', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-labelling-service',
      'text',
      array(
        'id'            => 'label-service-provider',
        'title'         => __( 'Service Provider', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-labelling-service',
      'text',
      array(
        'id'            => 'label-any',
        'title'         => __( 'Any (Service Provider)', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_section(
      'ewd-uasp-settings',
      array_merge(
        array(
          'id'            => 'ewd-uasp-labelling-appointment',
          'title'         => __( 'Appointment', 'ultimate-appointment-scheduling' ),
          'tab'         => 'ewd-uasp-labelling-tab',
        ),
        $this->get_permission( 'labelling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-labelling-appointment',
      'text',
      array(
        'id'            => 'label-appointment-title',
        'title'         => __( '\'Appointment\' Title', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-labelling-appointment',
      'text',
      array(
        'id'            => 'label-appointment-date',
        'title'         => __( 'Appointment Date', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-labelling-appointment',
      'text',
      array(
        'id'            => 'label-find-appointment',
        'title'         => __( 'Find Appointments', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-labelling-appointment',
      'text',
      array(
        'id'            => 'label-book-appointment',
        'title'         => __( 'Book Appointment', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-labelling-appointment',
      'text',
      array(
        'id'            => 'label-pay-in-advance',
        'title'         => __( 'Pay in Advance', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-labelling-appointment',
      'text',
      array(
        'id'            => 'label-proceed-to-payment',
        'title'         => __( 'Proceed to Payment', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-labelling-appointment',
      'text',
      array(
        'id'            => 'label-select-time',
        'title'         => __( 'Select Time', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-labelling-appointment',
      'text',
      array(
        'id'            => 'label-click-select-date',
        'title'         => __( 'Click here to select a date', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-labelling-appointment',
      'text',
      array(
        'id'            => 'label-image-number',
        'title'         => __( 'Image Number', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    return $sap;
  }

  // "Styling" Tab
  public function ewd_uasp_settings_styling_tab( $sap, $ewduaspSettings ) {
    
    $sap->add_section(
      'ewd-uasp-settings',
      array_merge(
        array(
          'id'            => 'ewd-uasp-styling-sign-up',
          'title'         => __( 'Sign Up Form', 'ultimate-appointment-scheduling' ),
          'tab'         => 'ewd-uasp-styling-tab',
        ),
        $this->get_permission( 'styling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-sign-up',
      'colorpicker',
      array(
        'id'      => 'styling-signup-title-font-color',
        'title'     => __( 'Title Font Color', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-sign-up',
      'text',
      array(
        'id'            => 'styling-signup-title-font',
        'title'         => __( 'Title Font Family', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-sign-up',
      'text',
      array(
        'id'            => 'styling-signup-title-font-size',
        'title'         => __( 'Title Font Size', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-sign-up',
      'text',
      array(
        'id'            => 'styling-signup-label-font',
        'title'         => __( 'Label Font Family', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-sign-up',
      'text',
      array(
        'id'            => 'styling-signup-label-font-size',
        'title'         => __( 'Label Font Size', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-sign-up',
      'colorpicker',
      array(
        'id'      => 'styling-signup-block-color',
        'title'     => __( 'Block Color', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-sign-up',
      'text',
      array(
        'id'            => 'styling-signup-block-margin',
        'title'         => __( 'Block Margin', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-sign-up',
      'text',
      array(
        'id'            => 'styling-signup-block-padding',
        'title'         => __( 'Block Padding', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_section(
      'ewd-uasp-settings',
      array_merge(
        array(
          'id'            => 'ewd-uasp-styling-service',
          'title'         => __( 'Choose Service Form', 'ultimate-appointment-scheduling' ),
          'tab'         => 'ewd-uasp-styling-tab',
        ),
        $this->get_permission( 'styling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-service',
      'colorpicker',
      array(
        'id'      => 'styling-service-title-font-color',
        'title'     => __( 'Title Font Color', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-service',
      'text',
      array(
        'id'            => 'styling-service-title-font',
        'title'         => __( 'Title Font Family', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-service',
      'text',
      array(
        'id'            => 'styling-service-title-font-size',
        'title'         => __( 'Title Font Size', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-service',
      'text',
      array(
        'id'            => 'styling-service-label-font',
        'title'         => __( 'Label Font Family', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-service',
      'text',
      array(
        'id'            => 'styling-service-label-font-size',
        'title'         => __( 'Label Font Size', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-service',
      'colorpicker',
      array(
        'id'      => 'styling-service-block-color',
        'title'     => __( 'Block Color', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-service',
      'text',
      array(
        'id'            => 'styling-service-block-margin',
        'title'         => __( 'Block Margin', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-service',
      'text',
      array(
        'id'            => 'styling-service-block-padding',
        'title'         => __( 'Block Padding', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_section(
      'ewd-uasp-settings',
      array_merge(
        array(
          'id'            => 'ewd-uasp-styling-appointment',
          'title'         => __( 'Find Appointment Form', 'ultimate-appointment-scheduling' ),
          'tab'         => 'ewd-uasp-styling-tab',
        ),
        $this->get_permission( 'styling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-appointment',
      'colorpicker',
      array(
        'id'      => 'styling-appointment-title-font-color',
        'title'     => __( 'Title Font Color', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-appointment',
      'text',
      array(
        'id'            => 'styling-appointment-title-font',
        'title'         => __( 'Title Font Family', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-appointment',
      'text',
      array(
        'id'            => 'styling-appointment-title-font-size',
        'title'         => __( 'Title Font Size', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-appointment',
      'text',
      array(
        'id'            => 'styling-appointment-label-font',
        'title'         => __( 'Label Font Family', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-appointment',
      'text',
      array(
        'id'            => 'styling-appointment-label-font-size',
        'title'         => __( 'Label Font Size', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-appointment',
      'colorpicker',
      array(
        'id'      => 'styling-appointment-block-color',
        'title'     => __( 'Block Color', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-appointment',
      'text',
      array(
        'id'            => 'styling-appointment-block-margin',
        'title'         => __( 'Block Margin', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-appointment',
      'text',
      array(
        'id'            => 'styling-appointment-block-padding',
        'title'         => __( 'Block Padding', 'ultimate-appointment-scheduling' ),
        'description' => ''
      )
    );

    $sap->add_section(
      'ewd-uasp-settings',
      array_merge(
        array(
          'id'            => 'ewd-uasp-styling-calendar',
          'title'         => __( 'Calendar Styling', 'ultimate-appointment-scheduling' ),
          'tab'         => 'ewd-uasp-styling-tab',
        ),
        $this->get_permission( 'styling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-calendar',
      'text',
      array(
        'id'      => 'styling-calendar-appt-font',
        'title'     => __( 'Appointment Font Family', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-calendar',
      'text',
      array(
        'id'      => 'styling-calendar-appt-font-size',
        'title'     => __( 'Appointment Font Size', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-calendar',
      'colorpicker',
      array(
        'id'      => 'styling-calendar-appt-color',
        'title'     => __( 'Appointment Font Color', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-calendar',
      'colorpicker',
      array(
        'id'      => 'styling-calendar-appt-bg-color',
        'title'     => __( 'Appointment Background Color', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-calendar',
      'colorpicker',
      array(
        'id'      => 'styling-calendar-appt-border-color',
        'title'     => __( 'Appointment Border Color', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-calendar',
      'colorpicker',
      array(
        'id'      => 'styling-calendar-appt-selected-bg-color',
        'title'     => __( 'Selected Appointment Background Color', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-calendar',
      'colorpicker',
      array(
        'id'      => 'styling-calendar-appt-selected-border-color',
        'title'     => __( 'Selected Appointment Border Color', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_section(
      'ewd-uasp-settings',
      array_merge(
        array(
          'id'            => 'ewd-uasp-styling-button',
          'title'         => __( 'Button Styling', 'ultimate-appointment-scheduling' ),
          'tab'         => 'ewd-uasp-styling-tab',
        ),
        $this->get_permission( 'styling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-button',
      'colorpicker',
      array(
        'id'      => 'styling-button-font-color',
        'title'     => __( 'Text Color', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-button',
      'colorpicker',
      array(
        'id'      => 'styling-button-color',
        'title'     => __( 'Background Color', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-button',
      'text',
      array(
        'id'      => 'styling-button-font',
        'title'     => __( 'Font Family', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-button',
      'text',
      array(
        'id'      => 'styling-button-font-size',
        'title'     => __( 'Font Size', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-button',
      'text',
      array(
        'id'      => 'styling-button-margin',
        'title'     => __( 'Margin', 'ultimate-appointment-scheduling' )
      )
    );

    $sap->add_setting(
      'ewd-uasp-settings',
      'ewd-uasp-styling-button',
      'text',
      array(
        'id'      => 'styling-button-padding',
        'title'     => __( 'Padding', 'ultimate-appointment-scheduling' )
      )
    );
    
    return $sap;
  }

}
}