<?php
if ( ! defined( 'ABSPATH' ) ) exit;

if ( !class_exists( 'ewdupcpEWDPHSettings' ) ) {
class ewdupcpEWDPHSettings {

  public function __construct() {

    add_filter( 'ewd_upcp_defaults', array( $this, 'ewd_upcp_settings_set_defaults' ), 10, 2 );

    add_filter( 'ewd_upcp_settings_page', array( $this, 'ewd_upcp_settings__non_premium_tabs' ), 10, 2 );

    add_filter( 'ewd_upcp_settings_page', array( $this, 'ewd_upcp_settings_premium_tab' ), 10, 2 );
    add_filter( 'ewd_upcp_settings_page', array( $this, 'ewd_upcp_settings_woocommerce_tab' ), 10, 2 );
    add_filter( 'ewd_upcp_settings_page', array( $this, 'ewd_upcp_settings_seo_tab' ), 10, 2 );
    add_filter( 'ewd_upcp_settings_page', array( $this, 'ewd_upcp_settings_labelling_tab' ), 10, 2 );
    add_filter( 'ewd_upcp_settings_page', array( $this, 'ewd_upcp_settings_styling_tab' ), 10, 2 );
  }

  public function get_permission( $permission_type = '' ) {
    global $ewd_upcp_controller;
  
    $ewd_upcp_premium_permissions = array();
  
    if ( ! $ewd_upcp_controller->permissions->check_permission( $permission_type ) ) {
      $ewd_upcp_premium_permissions = array(
        'disabled'    => true,
        'disabled_image'=> '#',
        'purchase_link' => 'https://www.etoilewebdesign.com/plugins/ultimate-product-catalog/?utm_source=upcp_lockbox'
      );
    }
  
    return $ewd_upcp_premium_permissions;
  }

  public function ewd_upcp_settings_set_defaults( $defaults, $ewdupcpSettings ) {

    $defaults = array_merge(
      $defaults,
      array(
        // Any default which you are certain that won't be used for free version.
      )
    );
  
    return $defaults;
  }

  public function ewd_upcp_settings__non_premium_tabs( $sap, $ewdupcpSettings ) {

    return $sap;
  }

  // "Premium" Tab
  public function ewd_upcp_settings_premium_tab( $sap, $ewdupcpSettings ) {
    
    $sap->add_section(
      'ewd-upcp-settings',
      array_merge(
        array(
          'id'          => 'ewd-upcp-premium-product-page',
          'title'       => __( 'Product Page', 'ultimate-product-catalogue' ),
          'tab'         => 'ewd-upcp-premium-tab',
        ),
        $this->get_permission( 'premium' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-premium-product-page',
      'radio',
      array(
        'id'      => 'product-page',
        'title'     => __( 'Product Page Type', 'ultimate-product-catalogue' ),
        'description' => __( 'What style of product page should be used?', 'ultimate-product-catalogue' ),
        'options'   => array(
          'default'   => __( 'Default', 'ultimate-product-catalogue' ),
          'tabbed'    => __( 'Tabbed', 'ultimate-product-catalogue' ),
          'shop_style'  => __( 'Shop Style', 'ultimate-product-catalogue' ),
          'custom'    => __( 'Custom', 'ultimate-product-catalogue' ),
          'large'     => __( 'Custom - Large Screen Only', 'ultimate-product-catalogue' ),
        ),
        'default'   => $ewdupcpSettings->defaults['product-page']
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-premium-product-page',
      'radio',
      array(
        'id'      => 'product-image-lightbox',
        'title'     => __( 'Lightbox', 'ultimate-product-catalogue' ),
        'description' => __( 'Should a lightbox pop-up on the product page when an image is clicked? Want to customize this lightbox? Install the <a href=\'https://wordpress.org/plugins/ultimate-lightbox/\' target=\'_blank\'>Ultimate Lightbox</a> plugin and you can switch the lightbox colors, controls, behaviour and more!', 'ultimate-product-catalogue' ),
        'options'   => array(
          'yes'     => __( 'Yes', 'ultimate-product-catalogue' ),
          'main'      => __( 'Main Image Only', 'ultimate-product-catalogue' ),
          'no'      => __( 'No', 'ultimate-product-catalogue' ),
        ),
        'default'   => $ewdupcpSettings->defaults['product-image-lightbox']
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-premium-product-page',
      'radio',
      array(
        'id'      => 'related-products',
        'title'     => __( 'Related Products', 'ultimate-product-catalogue' ),
        'description' => __( 'Should related products be created automatically, manually, or not at all?', 'ultimate-product-catalogue' ),
        'options'   => array(
          'automatic'   => __( 'Automatic', 'ultimate-product-catalogue' ),
          'manual'    => __( 'Manual', 'ultimate-product-catalogue' ),
          'none'      => __( 'None', 'ultimate-product-catalogue' ),
        ),
        'default'   => $ewdupcpSettings->defaults['related-products']
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-premium-product-page',
      'radio',
      array(
        'id'      => 'next-previous-products',
        'title'     => __( 'Next/Previous Products', 'ultimate-product-catalogue' ),
        'description' => __( 'Should next and previous products be displayed? Automatic takes the products with the next and previous IDs, if they exist.', 'ultimate-product-catalogue' ),
        'options'   => array(
          'automatic'   => __( 'Automatic', 'ultimate-product-catalogue' ),
          'manual'    => __( 'Manual', 'ultimate-product-catalogue' ),
          'none'      => __( 'None', 'ultimate-product-catalogue' ),
        ),
        'default'   => $ewdupcpSettings->defaults['next-previous-products']
      )
    );

    $sap->add_section(
      'ewd-upcp-settings',
      array_merge(
        array(
          'id'            => 'ewd-upcp-premium-catalog-page',
          'title'         => __( 'Catalog Page', 'ultimate-product-catalogue' ),
          'tab'         => 'ewd-upcp-premium-tab',
        ),
        $this->get_permission( 'premium' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-premium-catalog-page',
      'toggle',
      array(
        'id'      => 'lightbox-mode',
        'title'     => __( 'Lightbox Mode', 'ultimate-product-catalogue' ),
        'description' => __( 'Should a lightbox pop-up to display more information about products when they\'re clicked on?', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-premium-catalog-page',
      'toggle',
      array(
        'id'      => 'infinite-scroll',
        'title'     => __( 'Infinite Scroll', 'ultimate-product-catalogue' ),
        'description' => __( 'Should products load as a user scrolls down the page, instead of using the pagination system?', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-premium-catalog-page',
      'text',
      array(
        'id'            => 'products-per-page',
        'title'         => __( 'Products per Page', 'ultimate-product-catalogue' ),
        'description' => __( 'Set the maximum number of products per page for your catalogs', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-premium-catalog-page',
      'radio',
      array(
        'id'      => 'pagination-location',
        'title'     => __( 'Pagination Location', 'ultimate-product-catalogue' ),
        'description' => __( 'Set the location of pagination controls for your catalogs', 'ultimate-product-catalogue' ),
        'options'   => array(
          'top'     => __( 'Top', 'ultimate-product-catalogue' ),
          'bottom'    => __( 'Bottom', 'ultimate-product-catalogue' ),
          'both'      => __( 'Both', 'ultimate-product-catalogue' ),
        ),
        'default'   => $ewdupcpSettings->defaults['pagination-location']
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-premium-catalog-page',
      'checkbox',
      array(
        'id'      => 'product-sort',
        'title'     => __( 'Product Sorting', 'ultimate-product-catalogue' ),
        'description' => __( 'Select which sorting options are available in the \'Sort By\' box (\'Review Rating\' requires \'Ultimate Reviews\' to be installed)', 'ultimate-product-catalogue' ),
        'options'   => array(
          'price'     => __( 'Price', 'ultimate-product-catalogue' ),
          'name'      => __( 'Name', 'ultimate-product-catalogue' ),
          'rating'    => __( 'Review Ratings', 'ultimate-product-catalogue' ),
          'date'      => __( 'Date Added', 'ultimate-product-catalogue' ),
        )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-premium-catalog-page',
      'toggle',
      array(
        'id'      => 'disable-toggle-sidebar-on-mobile',
        'title'     => __( 'Disable Drop Down Sidebar Toggle on Mobile', 'ultimate-product-catalogue' ),
        'description' => __( 'Disable the \'Filter\' button, on mobile devices, that will display allowing the user to open the sidebar, with the sidebar hidden by default.', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_section(
      'ewd-upcp-settings',
      array_merge(
        array(
          'id'            => 'ewd-upcp-premium-features',
          'title'         => __( 'Features', 'ultimate-product-catalogue' ),
          'tab'         => 'ewd-upcp-premium-tab',
        ),
        $this->get_permission( 'premium' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-premium-features',
      'toggle',
      array(
        'id'      => 'product-inquiry-form',
        'title'     => __( 'Product Inquiry Form', 'ultimate-product-catalogue' ),
        'description' => __( 'Should a form be added to inquire about products on the product page (requires plugin \'WP Forms\' or \'Contact Form 7\')?', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-premium-features',
      'toggle',
      array(
        'id'      => 'product-inquiry-cart',
        'title'     => __( 'Product Inquiry Cart', 'ultimate-product-catalogue' ),
        'description' => __( 'Should users be able to inquire about multiple products at once from the main catalog page (requires plugin \'WP Forms\' or \'Contact Form 7\')?', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-premium-features',
      'radio',
      array(
        'id'      => 'product-inquiry-plugin',
        'title'     => __( 'Inquiry Plugin', 'ultimate-product-catalogue' ),
        'description' => __( 'Which contact form plugin should be used?', 'ultimate-product-catalogue' ),
        'options'   => array(
          'cf7'     => __( 'Contact Form 7', 'ultimate-product-catalogue' ),
          'wpforms'   => __( 'WP Forms', 'ultimate-product-catalogue' ),
        ),
        'default'   => $ewdupcpSettings->defaults['product-inquiry-plugin']
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-premium-features',
      'toggle',
      array(
        'id'      => 'product-reviews',
        'title'     => __( 'Product Reviews', 'ultimate-product-catalogue' ),
        'description' => __( 'Should reviews be displayed for products on the \'Tabbed Layout\' product page (requires plugin \'Ultimate Reviews\')?', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-premium-features',
      'toggle',
      array(
        'id'      => 'catalog-display-reviews',
        'title'     => __( 'Reviews in Main Catalog', 'ultimate-product-catalogue' ),
        'description' => __( 'Should a product\'s review rating be displayed on the main catalog page? (requires plugin \'<a href=\'https://wordpress.org/plugins/ultimate-reviews/\'>Ultimate Reviews</a>\')?', 'ultimate-product-catalogue' ),
        'conditional_on'    => 'product-reviews',
        'conditional_on_value'  => true
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-premium-features',
      'toggle',
      array(
        'id'      => 'product-faqs',
        'title'     => __( 'Product FAQs', 'ultimate-product-catalogue' ),
        'description' => __( 'Should FAQs be displayed for products on the Tabbed or Shop Style layout product page (requires Ultimate FAQs plugin)?', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-premium-features',
      'toggle',
      array(
        'id'      => 'product-comparison',
        'title'     => __( 'Allow Product Comparison', 'ultimate-product-catalogue' ),
        'description' => __( 'Should visitors be able to compare products side by side by clicking on the comparison link?', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-premium-features',
      'toggle',
      array(
        'id'      => 'hide-blank-custom-fields',
        'title'     => __( 'Hide Blank Custom Fields', 'ultimate-product-catalogue' ),
        'description' => __( 'Should custom fields be hidden when they are empty?', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-premium-features',
      'toggle',
      array(
        'id'      => 'disable-custom-field-conversion',
        'title'     => __( 'Disable Custom Slugs Conversion', 'ultimate-product-catalogue' ),
        'description' => __( 'Should converting of custom field slugs used in the product descriptions be disabled?', 'ultimate-product-catalogue' )
      )
    );

    return $sap;
  }

  // "WooCommerce" Tab
  public function ewd_upcp_settings_woocommerce_tab( $sap, $ewdupcpSettings ) {
    
    $sap->add_section(
      'ewd-upcp-settings',
      array_merge(
        array(
          'id'            => 'ewd-upcp-woocommerce',
          'title'         => __( 'Settings', 'ultimate-product-catalogue' ),
          'tab'         => 'ewd-upcp-woocommerce-tab',
        ),
        $this->get_permission( 'woocommerce' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-woocommerce',
      'toggle',
      array(
        'id'      => 'woocommerce-sync',
        'title'     => __( 'WooCommerce Sync', 'ultimate-product-catalogue' ),
        'description' => __( 'Should UPCP products be exported to WooCommerce, and WooCommerce products imported into UPCP? Products edited in one plugin will also be edited in the other while this is enabled. Using WooCommerce attributes or UPCP custom fields and want those to sync as well? Check out our blog post on how to <a href=\'http://www.etoilewebdesign.com/product-catalog-woocommerce-sync-tips/\'>get the best syncing results for attributes</a>.', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-woocommerce',
      'toggle',
      array(
        'id'      => 'woocommerce-disable-cart-count',
        'title'     => __( 'Disable Cart Item Count', 'ultimate-product-catalogue' ),
        'description' => __( 'Should the number of items in a visitors shopping cart be hidden?', 'ultimate-product-catalogue' ),
        'conditional_on'    => 'woocommerce-sync',
        'conditional_on_value'  => true
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-woocommerce',
      'toggle',
      array(
        'id'      => 'woocommerce-checkout',
        'title'     => __( 'WooCommerce Checkout', 'ultimate-product-catalogue' ),
        'description' => __( 'Should checkout be allowed, using the standard WooCommerce checkout? WARNING: WooCommerce sync must be enabled.', 'ultimate-product-catalogue' ),
        'conditional_on'    => 'woocommerce-sync',
        'conditional_on_value'  => true
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-woocommerce',
      'radio',
      array(
        'id'      => 'woocommerce-cart-page',
        'title'     => __( 'WooCommerce Cart Page', 'ultimate-product-catalogue' ),
        'description' => __( 'What WooCommerce page should visitors be sent to when they click the \'Checkout\' cart button? WARNING: WooCommerce sync must be enabled.', 'ultimate-product-catalogue' ),
        'options'   => array(
          'cart'      => __( 'Cart', 'ultimate-product-catalogue' ),
          'checkout'    => __( 'Checkout', 'ultimate-product-catalogue' ),
        ),
        'default'   => $ewdupcpSettings->defaults['woocommerce-cart-page'],
        'conditional_on'    => 'woocommerce-sync',
        'conditional_on_value'  => true
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-woocommerce',
      'toggle',
      array(
        'id'      => 'woocommerce-product-page',
        'title'     => __( 'WooCommerce Product Page', 'ultimate-product-catalogue' ),
        'description' => __( 'Should visitors be redirected to WooCommerce product pages instead of UPCP product pages when clicking on a product? <br/>Having trouble with this setting? Try changing the \'Permalink Base\' setting in the \'SEO\' tab.', 'ultimate-product-catalogue' ),
        'conditional_on'    => 'woocommerce-sync',
        'conditional_on_value'  => true
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-woocommerce',
      'toggle',
      array(
        'id'      => 'woocommerce-back-link',
        'title'     => __( 'WooCommerce Back Link', 'ultimate-product-catalogue' ),
        'description' => __( 'Should the WooCommerce product page breadcrumbs be replaced with a \'Back to Catalog\' link when coming directly from the catalog page?', 'ultimate-product-catalogue' ),
        'conditional_on'    => 'woocommerce-sync',
        'conditional_on_value'  => true
      )
    );

    return $sap;
  }

  // "SEO" Tab
  public function ewd_upcp_settings_seo_tab( $sap, $ewdupcpSettings ) {
    
    $sap->add_section(
      'ewd-upcp-settings',
      array_merge(
        array(
          'id'            => 'ewd-upcp-seo',
          'title'         => __( 'SEO Options', 'ultimate-product-catalogue' ),
          'tab'         => 'ewd-upcp-seo-tab',
        ),
        $this->get_permission( 'seo' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-seo',
      'toggle',
      array(
        'id'      => 'pretty-permalinks',
        'title'     => __( 'Pretty Permalinks', 'ultimate-product-catalogue' ),
        'description' => __( 'Should the plugin create SEO-friendly product page URLs? (Make sure product slugs have been filled in)', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-seo',
      'text',
      array(
        'id'            => 'permalink-base',
        'title'         => __( 'Permalink Base', 'ultimate-product-catalogue' ),
        'description' => __( 'Set the permalink base for pretty permalinks for your product pages, if you want something other than \'product\' as a permalink base. You may need to re-save your permalink structure for this to take effect.', 'ultimate-product-catalogue' ),
        'conditional_on'    => 'pretty-permalinks',
        'conditional_on_value'  => true
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-seo',
      'text',
      array(
        'id'            => 'product-page-permalink-base',
        'title'         => __( 'Post Type Permalink Base', 'ultimate-product-catalogue' ),
        'description' => __( 'Set the permalink base for the built-in WordPress single post page for this plugin\'s product post type, if you would like it to be something other than \'upcp_product.\' You may need to re-save your permalink structure for this to take effect.', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-seo',
      'radio',
      array(
        'id'      => 'seo-plugin',
        'title'     => __( 'SEO By Yoast Integration', 'ultimate-product-catalogue' ),
        'description' => __( 'Should descriptions and titles for product pages be added using Yoast?', 'ultimate-product-catalogue' ),
        'options'   => array(
          'yoast'     => __( 'Yes', 'ultimate-product-catalogue' ),
          'none'      => __( 'No', 'ultimate-product-catalogue' ),
        ),
        'default'   => $ewdupcpSettings->defaults['seo-plugin']
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-seo',
      'radio',
      array(
        'id'      => 'seo-integration',
        'title'     => __( 'Description Handling', 'ultimate-product-catalogue' ),
        'description' => __( 'If using Yoast, should the page description be added to or replaced?', 'ultimate-product-catalogue' ),
        'options'   => array(
          'add'     => __( 'Add', 'ultimate-product-catalogue' ),
          'replace'   => __( 'Replace', 'ultimate-product-catalogue' ),
        ),
        'default'   => $ewdupcpSettings->defaults['seo-integration'],
        'conditional_on'    => 'seo-plugin',
        'conditional_on_value'  => 'yoast'
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-seo',
      'text',
      array(
        'id'            => 'seo-title',
        'title'         => __( 'SEO Title', 'ultimate-product-catalogue' ),
        'description' => __( 'What should the page title be set to? Can use [page-title], [product-name], [category-name], [subcategory_name] to substitute those in the title.', 'ultimate-product-catalogue' )
      )
    );

    return $sap;
  }

  // "Labelling" Tab
  public function ewd_upcp_settings_labelling_tab( $sap, $ewdupcpSettings ) {
    
    $sap->add_section(
      'ewd-upcp-settings',
      array_merge(
        array(
          'id'            => 'ewd-upcp-labelling-sidebar',
          'title'         => __( 'Sidebar', 'ultimate-product-catalogue' ),
          'tab'         => 'ewd-upcp-labelling-tab',
        ),
        $this->get_permission( 'labelling' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-sidebar',
      'text',
      array(
        'id'            => 'label-categories',
        'title'         => __( 'Categories Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-sidebar',
      'text',
      array(
        'id'            => 'label-subcategories',
        'title'         => __( 'Sub-Categories Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-sidebar',
      'text',
      array(
        'id'            => 'label-tags',
        'title'         => __( 'Tags Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-sidebar',
      'text',
      array(
        'id'            => 'label-show-all',
        'title'         => __( 'Clear All Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-sidebar',
      'text',
      array(
        'id'            => 'label-sort-by',
        'title'         => __( 'Sort By Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-sidebar',
      'text',
      array(
        'id'            => 'label-price-ascending',
        'title'         => __( 'Price (Ascending) Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-sidebar',
      'text',
      array(
        'id'            => 'label-price-descending',
        'title'         => __( 'Price (Descending) Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-sidebar',
      'text',
      array(
        'id'            => 'label-name-ascending',
        'title'         => __( 'Name (Ascending) Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-sidebar',
      'text',
      array(
        'id'            => 'label-name-descending',
        'title'         => __( 'Name (Descending) Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-sidebar',
      'text',
      array(
        'id'            => 'label-product-name-search',
        'title'         => __( 'Product Search (Product Name) Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-sidebar',
      'text',
      array(
        'id'            => 'label-product-name-text',
        'title'         => __( 'Search Placeholder Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-sidebar',
      'text',
      array(
        'id'            => 'label-price-filter',
        'title'         => __( 'Price Filter/Slider Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_section(
      'ewd-upcp-settings',
      array_merge(
        array(
          'id'            => 'ewd-upcp-labelling-catalog',
          'title'         => __( 'Catalog', 'ultimate-product-catalogue' ),
          'tab'         => 'ewd-upcp-labelling-tab',
        ),
        $this->get_permission( 'labelling' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-catalog',
      'text',
      array(
        'id'            => 'label-updating-results',
        'title'         => __( 'Updating Results Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-catalog',
      'text',
      array(
        'id'            => 'label-no-results-found',
        'title'         => __( 'No Products Found Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-catalog',
      'text',
      array(
        'id'            => 'label-products-pagination',
        'title'         => __( 'Products Pagination Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-catalog',
      'text',
      array(
        'id'            => 'label-page',
        'title'         => __( '\'Page\' Pagination Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-catalog',
      'text',
      array(
        'id'            => 'label-pagination-of',
        'title'         => __( '\'Of\' Pagination Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-catalog',
      'text',
      array(
        'id'            => 'label-read-more',
        'title'         => __( 'Read More Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-catalog',
      'text',
      array(
        'id'            => 'label-compare',
        'title'         => __( 'Compare Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-catalog',
      'text',
      array(
        'id'            => 'label-side-by-side',
        'title'         => __( 'Side by side Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-catalog',
      'text',
      array(
        'id'            => 'label-sale',
        'title'         => __( 'Sale Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-catalog',
      'text',
      array(
        'id'            => 'label-inquire-button',
        'title'         => __( 'Inquire Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-catalog',
      'text',
      array(
        'id'            => 'label-add-to-cart-button',
        'title'         => __( 'Add to Cart Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-catalog',
      'text',
      array(
        'id'            => 'label-send-inquiry',
        'title'         => __( 'Send Inquiry! Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-catalog',
      'text',
      array(
        'id'            => 'label-checkout',
        'title'         => __( 'Checkout! Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-catalog',
      'text',
      array(
        'id'            => 'label-empty-cart',
        'title'         => __( 'Empty cart Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-catalog',
      'text',
      array(
        'id'            => 'label-cart-items',
        'title'         => __( '\'%s items in cart\' Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_section(
      'ewd-upcp-settings',
      array_merge(
        array(
          'id'            => 'ewd-upcp-labelling-product-page',
          'title'         => __( 'Product Page', 'ultimate-product-catalogue' ),
          'tab'         => 'ewd-upcp-labelling-tab',
        ),
        $this->get_permission( 'labelling' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-product-page',
      'text',
      array(
        'id'            => 'label-back-to-catalog',
        'title'         => __( 'Catalog Label (in breadcrumbs)', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-product-page',
      'text',
      array(
        'id'            => 'label-product-details-tab',
        'title'         => __( 'Product Details Tab Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-product-page',
      'text',
      array(
        'id'            => 'label-additional-info-tab',
        'title'         => __( 'Additional Information Tab Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-product-page',
      'text',
      array(
        'id'            => 'label-contact-form-tab',
        'title'         => __( 'Contact Us Tab Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-product-page',
      'text',
      array(
        'id'            => 'label-product-inquiry-form-title',
        'title'         => __( 'Product Inquiry Form Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-product-page',
      'text',
      array(
        'id'            => 'label-customer-reviews-tab',
        'title'         => __( 'Customer Reviews Tab Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-product-page',
      'text',
      array(
        'id'            => 'label-related-products',
        'title'         => __( 'Related Products Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-product-page',
      'text',
      array(
        'id'            => 'label-next-product',
        'title'         => __( 'Next Product Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-product-page',
      'text',
      array(
        'id'            => 'label-previous-product',
        'title'         => __( 'Previous Product Label', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-product-page',
      'text',
      array(
        'id'            => 'label-product-page-category',
        'title'         => __( 'Category Label (Product page)', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-product-page',
      'text',
      array(
        'id'            => 'label-product-page-subcategory',
        'title'         => __( 'Sub-category Label (Product page)', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-labelling-product-page',
      'text',
      array(
        'id'            => 'label-product-page-tags',
        'title'         => __( 'Tags Label (Product page)', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    return $sap;
  }

  // "Styling" Tab
  public function ewd_upcp_settings_styling_tab( $sap, $ewdupcpSettings ) {
    
    $sap->add_section(
      'ewd-upcp-settings',
      array_merge(
        array(
          'id'            => 'ewd-upcp-styling-catalog',
          'title'         => __( 'Catalog', 'ultimate-product-catalogue' ),
          'tab'         => 'ewd-upcp-styling-tab',
        ),
        $this->get_permission( 'styling' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-catalog',
      'toggle',
      array(
        'id'      => 'styling-fixed-thumbnail-size',
        'title'     => __( 'Fixed Thumbnail Size', 'ultimate-product-catalogue' ),
        'description' => __( 'Enabling this will make all thumbnails the same size, cropping the image if necessary.', 'ultimate-product-catalogue' ),
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-catalog',
      'radio',
      array(
        'id'      => 'styling-catalog-skin',
        'title'     => __( 'Catalog Style', 'ultimate-product-catalogue' ),
        'description' => __( 'Selects a layout style for your catalog.', 'ultimate-product-catalogue' ),
        'options'   => array(
          'default'     => __( 'Default', 'ultimate-product-catalogue' ),
          'main-block'    => __( 'Block', 'ultimate-product-catalogue' ),
          'main-hover'    => __( 'Minimalist/Hover', 'ultimate-product-catalogue' ),
        ),
        'default'   => $ewdupcpSettings->defaults['styling-catalog-skin']
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-catalog',
      'radio',
      array(
        'id'      => 'styling-category-heading-style',
        'title'     => __( 'Category Heading Style', 'ultimate-product-catalogue' ),
        'description' => __( 'Selects a style for the category headings in your catalog views.', 'ultimate-product-catalogue' ),
        'options'   => array(
          'normal'    => __( 'Normal', 'ultimate-product-catalogue' ),
          'block'     => __( 'Block', 'ultimate-product-catalogue' ),
          'none'      => __( 'Not Displayed', 'ultimate-product-catalogue' ),
        ),
        'default'   => $ewdupcpSettings->defaults['styling-category-heading-style']
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-catalog',
      'radio',
      array(
        'id'      => 'styling-number-of-columns',
        'title'     => __( 'Thumbnail Number of Columns', 'ultimate-product-catalogue' ),
        'description' => __( 'Choose the number of columns you would like for the thumbnail view on larger screens.', 'ultimate-product-catalogue' ),
        'options'   => array(
          'three'     => __( '3', 'ultimate-product-catalogue' ),
          'four'      => __( '4', 'ultimate-product-catalogue' ),
          'five'      => __( '5', 'ultimate-product-catalogue' ),
        ),
        'default'   => $ewdupcpSettings->defaults['styling-number-of-columns']
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-catalog',
      'colorpicker',
      array(
        'id'      => 'styling-action-button-background-color',
        'title'     => __( 'Action Button Border/Background Hover', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-catalog',
      'colorpicker',
      array(
        'id'      => 'styling-action-button-text-color',
        'title'     => __( 'Action Button Text on Hover', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-catalog',
      'colorpicker',
      array(
        'id'      => 'styling-compare-button-background-color',
        'title'     => __( 'Compare Button Background', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-catalog',
      'colorpicker',
      array(
        'id'      => 'styling-compare-button-clicked-background-color',
        'title'     => __( 'Compare Button Clicked Background', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-catalog',
      'colorpicker',
      array(
        'id'      => 'styling-compare-button-text-color',
        'title'     => __( 'Compare Button Text', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-catalog',
      'colorpicker',
      array(
        'id'      => 'styling-compare-button-clicked-text-color',
        'title'     => __( 'Compare Button Clicked Text', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-catalog',
      'text',
      array(
        'id'            => 'styling-compare-button-font-size',
        'title'         => __( 'Compare Button Font Size', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-catalog',
      'colorpicker',
      array(
        'id'      => 'styling-sale-button-background-color',
        'title'     => __( 'Sale Button Background', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-catalog',
      'colorpicker',
      array(
        'id'      => 'styling-sale-button-text-color',
        'title'     => __( 'Sale Button Text', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-catalog',
      'text',
      array(
        'id'            => 'styling-sale-button-font-size',
        'title'         => __( 'Sale Button Font Size', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-catalog',
      'text',
      array(
        'id'            => 'styling-product-comparison-title-font-size',
        'title'         => __( 'Product Comparison Title Font Size', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-catalog',
      'colorpicker',
      array(
        'id'      => 'styling-product-comparison-title-font-color',
        'title'     => __( 'Product Comparison Title Font Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-catalog',
      'text',
      array(
        'id'            => 'styling-product-comparison-price-font-size',
        'title'         => __( 'Product Comparison Price Font Size', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-catalog',
      'colorpicker',
      array(
        'id'      => 'styling-product-comparison-price-font-color',
        'title'     => __( 'Product Comparison Price Font Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-catalog',
      'colorpicker',
      array(
        'id'      => 'styling-product-comparison-price-background-color',
        'title'     => __( 'Product Comparison Price Background Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_section(
      'ewd-upcp-settings',
      array_merge(
        array(
          'id'            => 'ewd-upcp-styling-thumbnail-view',
          'title'         => __( 'Thumbnail View', 'ultimate-product-catalogue' ),
          'tab'         => 'ewd-upcp-styling-tab',
        ),
        $this->get_permission( 'styling' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-thumbnail-view',
      'colorpicker',
      array(
        'id'      => 'styling-thumbnail-view-image-border-color',
        'title'     => __( 'Image Border Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-thumbnail-view',
      'text',
      array(
        'id'            => 'styling-thumbnail-view-box-min-height',
        'title'         => __( 'Box Min-Height', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-thumbnail-view',
      'text',
      array(
        'id'            => 'styling-thumbnail-view-box-max-height',
        'title'         => __( 'Box Max-Height', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-thumbnail-view',
      'text',
      array(
        'id'            => 'styling-thumbnail-view-box-padding',
        'title'         => __( 'Box Padding', 'ultimate-product-catalogue' ),
        'description' => 'Applies to the Block catalog style.'
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-thumbnail-view',
      'colorpicker',
      array(
        'id'      => 'styling-thumbnail-view-border-color',
        'title'     => __( 'Box Border Color', 'ultimate-product-catalogue' ),
        'description' => 'Applies to the Block catalog style.'
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-thumbnail-view',
      'text',
      array(
        'id'            => 'styling-thumbnail-view-title-font',
        'title'         => __( 'Product Title Font', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-thumbnail-view',
      'text',
      array(
        'id'            => 'styling-thumbnail-view-title-font-size',
        'title'         => __( 'Product Title Font Size', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-thumbnail-view',
      'colorpicker',
      array(
        'id'      => 'styling-thumbnail-view-title-font-color',
        'title'     => __( 'Product Title Font Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-thumbnail-view',
      'text',
      array(
        'id'            => 'styling-thumbnail-view-price-font',
        'title'         => __( 'Product Price Font', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-thumbnail-view',
      'text',
      array(
        'id'            => 'styling-thumbnail-view-price-font-size',
        'title'         => __( 'Product Price Font Size', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-thumbnail-view',
      'colorpicker',
      array(
        'id'      => 'styling-thumbnail-view-price-font-color',
        'title'     => __( 'Product Price Font Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-thumbnail-view',
      'colorpicker',
      array(
        'id'      => 'styling-thumbnail-view-background-color',
        'title'     => __( 'Product Background Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_section(
      'ewd-upcp-settings',
      array_merge(
        array(
          'id'            => 'ewd-upcp-styling-list-view',
          'title'         => __( 'List View', 'ultimate-product-catalogue' ),
          'tab'         => 'ewd-upcp-styling-tab',
        ),
        $this->get_permission( 'styling' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-list-view',
      'radio',
      array(
        'id'      => 'styling-list-view-click-action',
        'title'     => __( 'Product Click Action', 'ultimate-product-catalogue' ),
        'description' => __( 'When a product is clicked in list view, should the listing expand or should the user be sent to the product page?', 'ultimate-product-catalogue' ),
        'options'   => array(
          'expand'    => __( 'Expand', 'ultimate-product-catalogue' ),
          'product'   => __( 'Product Page', 'ultimate-product-catalogue' ),
        ),
        'default'   => $ewdupcpSettings->defaults['styling-list-view-click-action']
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-list-view',
      'colorpicker',
      array(
        'id'      => 'styling-list-view-image-border-color',
        'title'     => __( 'Image Border Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-list-view',
      'text',
      array(
        'id'            => 'styling-list-view-box-padding',
        'title'         => __( 'Box Padding', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-list-view',
      'text',
      array(
        'id'            => 'styling-list-view-box-margin-top',
        'title'         => __( 'Box Top Margin', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-list-view',
      'colorpicker',
      array(
        'id'      => 'styling-list-view-box-border-color',
        'title'     => __( 'Box Border Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-list-view',
      'text',
      array(
        'id'            => 'styling-list-view-title-font',
        'title'         => __( 'Product Title Font', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-list-view',
      'text',
      array(
        'id'            => 'styling-list-view-title-font-size',
        'title'         => __( 'Product Title Font Size', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-list-view',
      'colorpicker',
      array(
        'id'      => 'styling-list-view-title-font-color',
        'title'     => __( 'Product Title Font Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-list-view',
      'text',
      array(
        'id'            => 'styling-list-view-price-font',
        'title'         => __( 'Product Price Font', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-list-view',
      'text',
      array(
        'id'            => 'styling-list-view-price-font-size',
        'title'         => __( 'Product Price Font Size', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-list-view',
      'colorpicker',
      array(
        'id'      => 'styling-list-view-price-font-color',
        'title'     => __( 'Product Price Font Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_section(
      'ewd-upcp-settings',
      array_merge(
        array(
          'id'            => 'ewd-upcp-styling-detail-view',
          'title'         => __( 'Detail View', 'ultimate-product-catalogue' ),
          'tab'         => 'ewd-upcp-styling-tab',
        ),
        $this->get_permission( 'styling' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-detail-view',
      'colorpicker',
      array(
        'id'      => 'styling-detail-view-image-border-color',
        'title'     => __( 'Image Border Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-detail-view',
      'text',
      array(
        'id'            => 'styling-detail-view-box-padding',
        'title'         => __( 'Box Padding', 'ultimate-product-catalogue' ),
        'description' => 'Applies to the Block catalog style.'
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-detail-view',
      'text',
      array(
        'id'            => 'styling-detail-view-box-margin',
        'title'         => __( 'Box Margin', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-detail-view',
      'colorpicker',
      array(
        'id'      => 'styling-detail-view-box-background-color',
        'title'     => __( 'Box Background Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-detail-view',
      'colorpicker',
      array(
        'id'      => 'styling-detail-view-border-color',
        'title'     => __( 'Box Border Color', 'ultimate-product-catalogue' ),
        'description' => 'Applies to the Block catalog style.'
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-detail-view',
      'text',
      array(
        'id'            => 'styling-detail-view-title-font',
        'title'         => __( 'Product Title Font', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-detail-view',
      'text',
      array(
        'id'            => 'styling-detail-view-title-font-size',
        'title'         => __( 'Product Title Font Size', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-detail-view',
      'colorpicker',
      array(
        'id'      => 'styling-detail-view-title-font-color',
        'title'     => __( 'Product Title Font Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-detail-view',
      'text',
      array(
        'id'            => 'styling-detail-view-price-font',
        'title'         => __( 'Product Price Font', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-detail-view',
      'text',
      array(
        'id'            => 'styling-detail-view-price-font-size',
        'title'         => __( 'Product Price Font Size', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-detail-view',
      'colorpicker',
      array(
        'id'      => 'styling-detail-view-price-font-color',
        'title'     => __( 'Product Price Font Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_section(
      'ewd-upcp-settings',
      array_merge(
        array(
          'id'            => 'ewd-upcp-styling-sidebar',
          'title'         => __( 'Sidebar', 'ultimate-product-catalogue' ),
          'tab'         => 'ewd-upcp-styling-tab',
        ),
        $this->get_permission( 'styling' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-sidebar',
      'toggle',
      array(
        'id'      => 'styling-sidebar-title-collapse',
        'title'     => __( 'Collapsible Sidebar', 'ultimate-product-catalogue' ),
        'description' => __( 'Should sidebar titles collapse on-click?', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-sidebar',
      'toggle',
      array(
        'id'      => 'styling-sidebar-start-collapsed',
        'title'     => __( 'Sidebar Start Collapsed', 'ultimate-product-catalogue' ),
        'description' => __( 'Should sidebar content start collapsed? (Requires collapsible sidebar to be set to \'Yes\')', 'ultimate-product-catalogue' ),
        'conditional_on'    => 'styling-sidebar-title-collapse',
        'conditional_on_value'  => true
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-sidebar',
      'radio',
      array(
        'id'      => 'styling-sidebar-title-hover',
        'title'     => __( 'Sidebar Title Hover Effect', 'ultimate-product-catalogue' ),
        'description' => __( 'Which hover effect should apply on sidebar titles?', 'ultimate-product-catalogue' ),
        'options'   => array(
          'none'      => __( 'None', 'ultimate-product-catalogue' ),
          'underline'   => __( 'Underline', 'ultimate-product-catalogue' ),
        ),
        'default'   => $ewdupcpSettings->defaults['styling-sidebar-title-hover']
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-sidebar',
      'radio',
      array(
        'id'      => 'styling-sidebar-checkbox-style',
        'title'     => __( 'Checkbox Style', 'ultimate-product-catalogue' ),
        'description' => __( 'Which style should be applied to the catalog sidebar?', 'ultimate-product-catalogue' ),
        'options'   => array(
          'none'      => __( 'Default', 'ultimate-product-catalogue' ),
          'square'    => __( 'Checkmark', 'ultimate-product-catalogue' ),
          'minimalist'  => __( 'Minimalist', 'ultimate-product-catalogue' ),
          'block'     => __( 'Block', 'ultimate-product-catalogue' ),
        ),
        'default'   => $ewdupcpSettings->defaults['styling-sidebar-checkbox-style']
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-sidebar',
      'radio',
      array(
        'id'      => 'styling-sidebar-categories-control-type',
        'title'     => __( 'Categories Control Type', 'ultimate-product-catalogue' ),
        'description' => __( 'Which type of control should be used to filter categories?', 'ultimate-product-catalogue' ),
        'options'   => array(
          'checkbox'    => __( 'Checkbox', 'ultimate-product-catalogue' ),
          'radio'     => __( 'Radio', 'ultimate-product-catalogue' ),
          'dropdown'    => __( 'Dropdown', 'ultimate-product-catalogue' ),
        ),
        'default'   => $ewdupcpSettings->defaults['styling-sidebar-categories-control-type']
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-sidebar',
      'radio',
      array(
        'id'      => 'styling-sidebar-subcategories-control-type',
        'title'     => __( 'Sub-Categories Control Type', 'ultimate-product-catalogue' ),
        'description' => __( 'Which type of control should be used to filter sub-categories?', 'ultimate-product-catalogue' ),
        'options'   => array(
          'checkbox'    => __( 'Checkbox', 'ultimate-product-catalogue' ),
          'radio'     => __( 'Radio', 'ultimate-product-catalogue' ),
          'dropdown'    => __( 'Dropdown', 'ultimate-product-catalogue' ),
        ),
        'default'   => $ewdupcpSettings->defaults['styling-sidebar-subcategories-control-type']
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-sidebar',
      'radio',
      array(
        'id'      => 'styling-sidebar-tags-control-type',
        'title'     => __( 'Tags Control Type', 'ultimate-product-catalogue' ),
        'description' => __( 'Which type of control should be used to filter tags?', 'ultimate-product-catalogue' ),
        'options'   => array(
          'checkbox'    => __( 'Checkbox', 'ultimate-product-catalogue' ),
          'radio'     => __( 'Radio', 'ultimate-product-catalogue' ),
          'dropdown'    => __( 'Dropdown', 'ultimate-product-catalogue' ),
        ),
        'default'   => $ewdupcpSettings->defaults['styling-sidebar-tags-control-type']
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-sidebar',
      'ordering-table',
      array(
        'id'            => 'styling-sidebar-items-order',
        'title'         => __( 'Sidebar Items Order', 'ultimate-product-catalogue' ),
        'description'   => __( 'What order should the filtering controls appear in?', 'ultimate-product-catalogue' ), 
        'items'         => $ewdupcpSettings->get_setting( 'styling-sidebar-items-order' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-sidebar',
      'text',
      array(
        'id'            => 'styling-sidebar-header-font',
        'title'         => __( 'Sidebar Title Font', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-sidebar',
      'text',
      array(
        'id'            => 'styling-sidebar-header-font-size',
        'title'         => __( 'Sidebar Title Font Size', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-sidebar',
      'colorpicker',
      array(
        'id'      => 'styling-sidebar-header-font-color',
        'title'     => __( 'Sidebar Title Font Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-sidebar',
      'text',
      array(
        'id'            => 'styling-sidebar-header-font-weight',
        'title'         => __( 'Sidebar Title Font Weight', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-sidebar',
      'text',
      array(
        'id'            => 'styling-sidebar-checkbox-font',
        'title'         => __( 'Sidebar Checkbox Font', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-sidebar',
      'text',
      array(
        'id'            => 'styling-sidebar-checkbox-font-size',
        'title'         => __( 'Sidebar Checkbox Font Size', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-sidebar',
      'colorpicker',
      array(
        'id'      => 'styling-sidebar-checkbox-font-color',
        'title'     => __( 'Sidebar Checkbox Font Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-sidebar',
      'text',
      array(
        'id'            => 'styling-sidebar-checkbox-font-weight',
        'title'         => __( 'Sidebar Checkbox Font Weight', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_section(
      'ewd-upcp-settings',
      array_merge(
        array(
          'id'            => 'ewd-upcp-styling-product-page',
          'title'         => __( 'Product Page', 'ultimate-product-catalogue' ),
          'tab'         => 'ewd-upcp-styling-tab',
        ),
        $this->get_permission( 'styling' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-product-page',
      'text',
      array(
        'id'            => 'styling-breadcrumbs-font',
        'title'         => __( 'Breadcrumbs Font', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-product-page',
      'text',
      array(
        'id'            => 'styling-breadcrumbs-font-size',
        'title'         => __( 'Breadcrumbs Font Size', 'ultimate-product-catalogue' ),
        'description' => ''
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-product-page',
      'colorpicker',
      array(
        'id'      => 'styling-breadcrumbs-font-color',
        'title'     => __( 'Breadcrumbs Text Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-product-page',
      'colorpicker',
      array(
        'id'      => 'styling-breadcrumbs-font-hover-color',
        'title'     => __( 'Breadcrumbs Text Hover Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_section(
      'ewd-upcp-settings',
      array_merge(
        array(
          'id'            => 'ewd-upcp-styling-pagination',
          'title'         => __( 'Pagination', 'ultimate-product-catalogue' ),
          'tab'         => 'ewd-upcp-styling-tab',
        ),
        $this->get_permission( 'styling' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-pagination',
      'colorpicker',
      array(
        'id'      => 'styling-pagination-background-color',
        'title'     => __( 'Background Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-pagination',
      'colorpicker',
      array(
        'id'      => 'styling-pagination-text-color',
        'title'     => __( 'Text Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-pagination',
      'colorpicker',
      array(
        'id'      => 'styling-pagination-background-color-hover',
        'title'     => __( 'Background Hover Color', 'ultimate-product-catalogue' )
      )
    );

    $sap->add_setting(
      'ewd-upcp-settings',
      'ewd-upcp-styling-pagination',
      'colorpicker',
      array(
        'id'      => 'styling-pagination-text-color-hover',
        'title'     => __( 'Text Hover Color', 'ultimate-product-catalogue' )
      )
    );
    
    return $sap;
  }

}
}